/*
 * RepositoryContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.framework.repository;

import dk.apaq.framework.criteria.Criteria;

/**
 *  Event used when Repository's notifies of its actions.
 */
public class RepositoryEvent<BT, IDT> {

    private final Repository<BT, IDT> repository;

    /**
     * Create new basic RepositoryEvent.
     * @param crud The crud that generated the event.
     */
    public RepositoryEvent(Repository<BT, IDT> crud) {
        this.repository = crud;
    }

    /**
     * Retrieves the Repository that generated the wvent.
     * @return The crud.
     */
    public Repository<BT, IDT> getRepository() {
        return repository;
    }

    /**
     * Repository event for events that carries an id of the entity
     * @param <IDT> The id type
     * @param <BT> The bean type
     */
    public static class WithId<BT, IDT> extends RepositoryEvent<BT, IDT> {

        private final IDT id;

        /**
         * Create new RepositoryEvent with id.
         * @param crud The Repository generating the event.
         * @param id The id of the entity.
         */
        public WithId(Repository<BT, IDT> crud, IDT id) {
            super(crud);
            this.id = id;
        }

        /**
         * Retrieve the entity id.
         * @return The entity id.
         */
        public IDT getEntityId() {
            return id;
        }
    }
    
    /**
     * Repository event for events that carries the entity
     * @param <IDT> The id type
     * @param <BT> The bean type
     */
    public static class WithEntity<BT, IDT> extends RepositoryEvent<BT, IDT> {

        private final BT bean;


        /**
         * Create new RepositoryEvent with id and entity.
         * @param crud The Repository generating the event.
         * @param id The id of the entity.
         * @param entity The entity it self.
         */
        public WithEntity(Repository<BT, IDT> crud, BT bean) {
            super(crud);
            this.bean = bean;
        }

        /**
         * Retieves the entity.
         * @return
         */
        public BT getEntity() {
            return bean;
        }
    }

    /**
     * Repository event for events that carries an id of the entity and the entity itself
     * @param <IDT> The id type
     * @param <BT> The bean type
     */
    public static class WithIdAndEntity<BT, IDT> extends WithId<BT, IDT> {

        private final BT bean;


        /**
         * Create new RepositoryEvent with id and entity.
         * @param crud The Repository generating the event.
         * @param id The id of the entity.
         * @param entity The entity it self.
         */
        public WithIdAndEntity(Repository<BT, IDT> crud, IDT id, BT bean) {
            super(crud, id);
            this.bean = bean;
        }

        /**
         * Retieves the entity.
         * @return
         */
        public BT getEntity() {
            return bean;
        }
    }
    
    /**
     * Repository event for events that carries a criteriia for the data being listed
     * @param <IDT> The id type
     * @param <BT> The bean type
     */
    public static class List<BT, IDT> extends RepositoryEvent<BT, IDT> {

        private final Criteria criteria;

        /**
         * Create new RepositoryEvent with id and entity.
         * @param crud The Repository generating the event.
         * @param criteria The criteria.
         */
        public List(Repository<BT, IDT> crud, Criteria criteria) {
            super(crud);
            this.criteria = criteria;
        }

        public Criteria getCriteria() {
            return criteria;
        }

        

    }

}
